@@ -39,13 +39,14 @@ public:
     QXcbDrag(QXcbConnection *c);
     ~QXcbDrag();
 
-    virtual QMimeData *platformDropData() Q_DECL_OVERRIDE;
+    virtual QMimeData *platformDropData();
 
-    void startDrag() Q_DECL_OVERRIDE;
-    void cancel() Q_DECL_OVERRIDE;
-    void move(const QMouseEvent *me) Q_DECL_OVERRIDE;
-    void drop(const QMouseEvent *me) Q_DECL_OVERRIDE;
-    void endDrag() Q_DECL_OVERRIDE;
+
+    void startDrag();
+    void cancel();
+    void move(const QMouseEvent *me);
+    void drop(const QMouseEvent *me);
+    void endDrag();
 
     void handleEnter(QWindow *window, const xcb_client_message_event_t *event);
     void handlePosition(QWindow *w, const xcb_client_message_event_t *event);
@@ -57,13 +58,12 @@ public:
     void handleFinished(const xcb_client_message_event_t *event);
 
     bool dndEnable(QXcbWindow *win, bool on);
-    bool ownsDragObject() const Q_DECL_OVERRIDE;
 
     void updatePixmap();
     xcb_timestamp_t targetTime() { return target_time; }
 
 protected:
-    void timerEvent(QTimerEvent* e) Q_DECL_OVERRIDE;
+    void timerEvent(QTimerEvent* e);
 
 private:
     friend class QXcbDropData;
