@@ -8,7 +8,6 @@
 #include <xcb/xcb.h>
 #include <xcb/randr.h>
 
-#include <QtGui/private/qtguiglobal_p.h>
 #include "qxcbexport.h"
 #include <QHash>
 #include <QList>
@@ -19,24 +18,20 @@
 #include <QVarLengthArray>
 #include <qpa/qwindowsysteminterface.h>
 #include <QtCore/QLoggingCategory>
-#include <QtCore/private/qglobal_p.h>
-
-#include <cstdlib>
-#include <memory>
 
 // This is needed to make Qt compile together with XKB. xkb.h is using a variable
 // which is called 'explicit', this is a reserved keyword in c++
-#if QT_CONFIG(xkb)
+#ifndef QT_NO_XKB
 #define explicit dont_use_cxx_explicit
 #include <xcb/xkb.h>
 #undef explicit
 #endif
 
-#if QT_CONFIG(tabletevent)
+#ifndef QT_NO_TABLETEVENT
 #include <QTabletEvent>
 #endif
 
-#if QT_CONFIG(xinput2)
+#if XCB_USE_XINPUT2
 #include <X11/extensions/XI2.h>
 #ifdef XIScrollClass
 #define XCB_USE_XINPUT21    // XI 2.1 adds smooth scrolling support
@@ -44,19 +39,18 @@
 #define XCB_USE_XINPUT22    // XI 2.2 adds multi-point touch support
 #endif
 #endif
-#endif // QT_CONFIG(xinput2)
+struct XInput2TouchDeviceData;
+#endif // XCB_USE_XINPUT2
 
 struct xcb_randr_get_output_info_reply_t;
 
+//#define Q_XCB_DEBUG
+
 QT_BEGIN_NAMESPACE
 
 Q_DECLARE_LOGGING_CATEGORY(lcQpaXInput)
 Q_DECLARE_LOGGING_CATEGORY(lcQpaXInputDevices)
-Q_DECLARE_LOGGING_CATEGORY(lcQpaXInputEvents)
 Q_DECLARE_LOGGING_CATEGORY(lcQpaScreen)
-Q_DECLARE_LOGGING_CATEGORY(lcQpaEvents)
-Q_DECLARE_LOGGING_CATEGORY(lcQpaXcb)
-Q_DECLARE_LOGGING_CATEGORY(lcQpaPeeker)
 
 class QXcbVirtualDesktop;
 class QXcbScreen;
@@ -171,7 +165,6 @@ namespace QXcbAtom {
         _KDE_NET_WM_WINDOW_TYPE_OVERRIDE,
 
         _KDE_NET_WM_FRAME_STRUT,
-        _NET_FRAME_EXTENTS,
 
         _NET_STARTUP_INFO,
         _NET_STARTUP_INFO_BEGIN,
@@ -238,7 +231,6 @@ namespace QXcbAtom {
         AbsMTPositionY,
         AbsMTTouchMajor,
         AbsMTTouchMinor,
-        AbsMTOrientation,
         AbsMTPressure,
         AbsMTTrackingID,
         MaxContacts,
@@ -264,8 +256,6 @@ namespace QXcbAtom {
         _COMPIZ_DECOR_PENDING,
         _COMPIZ_DECOR_REQUEST,
         _COMPIZ_DECOR_DELETE_PIXMAP,
-        _COMPIZ_TOOLKIT_ACTION,
-        _GTK_LOAD_ICONTHEMES,
 
         NPredefinedAtoms,
 
@@ -283,7 +273,7 @@ class QXcbEventReader : public QThread
 public:
     QXcbEventReader(QXcbConnection *connection);
 
-    void run() override;
+    void run() Q_DECL_OVERRIDE;
 
     QXcbEventArray *lock();
     void unlock();
@@ -304,6 +294,9 @@ private:
     QMutex m_mutex;
     QXcbEventArray m_events;
     QXcbConnection *m_connection;
+
+    typedef xcb_generic_event_t * (*XcbPollForQueuedEventFunctionPointer)(xcb_connection_t *c);
+    XcbPollForQueuedEventFunctionPointer m_xcb_poll_for_queued_event;
 };
 
 class QXcbWindowEventListener
@@ -326,10 +319,8 @@ public:
     virtual void handleFocusInEvent(const xcb_focus_in_event_t *) {}
     virtual void handleFocusOutEvent(const xcb_focus_out_event_t *) {}
     virtual void handlePropertyNotifyEvent(const xcb_property_notify_event_t *) {}
-#if QT_CONFIG(xinput2)
-    virtual void handleXIMouseEvent(xcb_ge_event_t *, Qt::MouseEventSource = Qt::MouseEventNotSynthesized) {}
-    virtual void handleXIEnterLeave(xcb_ge_event_t *) {}
-#endif
+    virtual void handleXIMouseEvent(xcb_ge_event_t *) {}
+
     virtual QXcbWindow *toWindow() { return 0; }
 };
 
@@ -356,12 +347,9 @@ public:
     ~QXcbConnection();
 
     QXcbConnection *connection() const { return const_cast<QXcbConnection *>(this); }
-    bool isConnected() const;
 
-    const QList<QXcbVirtualDesktop *> &virtualDesktops() const { return m_virtualDesktops; }
     const QList<QXcbScreen *> &screens() const { return m_screens; }
     int primaryScreenNumber() const { return m_primaryScreenNumber; }
-    QXcbVirtualDesktop *primaryVirtualDesktop() const { return m_virtualDesktops.value(m_primaryScreenNumber); }
     QXcbScreen *primaryScreen() const;
 
     inline xcb_atom_t atom(QXcbAtom::Atom atom) const { return m_allAtoms[atom]; }
@@ -385,72 +373,69 @@ public:
 
     QXcbWMSupport *wmSupport() const { return m_wmSupport.data(); }
     xcb_window_t rootWindow();
-    xcb_window_t clientLeader();
 
     bool hasDefaultVisualId() const { return m_defaultVisualId != UINT_MAX; }
     xcb_visualid_t defaultVisualId() const { return m_defaultVisualId; }
 
-#if QT_CONFIG(xcb_xlib)
+#ifdef XCB_USE_XLIB
     void *xlib_display() const;
     void *createVisualInfoForDefaultVisualId() const;
 #endif
+
+#if defined(XCB_USE_XINPUT2)
+    void xi2Select(xcb_window_t window);
+#endif
+#ifdef XCB_USE_XINPUT21
+    bool isAtLeastXI21() const { return m_xi2Enabled && m_xi2Minor >= 1; }
+#else
+    bool isAtLeastXI21() const { return false; }
+#endif
+#ifdef XCB_USE_XINPUT22
+    bool isAtLeastXI22() const { return m_xi2Enabled && m_xi2Minor >= 2; }
+#else
+    bool isAtLeastXI22() const { return false; }
+#endif
+
     void sync();
 
     void handleXcbError(xcb_generic_error_t *error);
     void handleXcbEvent(xcb_generic_event_t *event);
-    void printXcbEvent(const QLoggingCategory &log, const char *message,
-                       xcb_generic_event_t *event) const;
 
     void addWindowEventListener(xcb_window_t id, QXcbWindowEventListener *eventListener);
     void removeWindowEventListener(xcb_window_t id);
     QXcbWindowEventListener *windowEventListenerFromId(xcb_window_t id);
     QXcbWindow *platformWindowFromId(xcb_window_t id);
 
+    xcb_generic_event_t *checkEvent(int type);
     template<typename T>
     inline xcb_generic_event_t *checkEvent(T &checker);
 
     typedef bool (*PeekFunc)(QXcbConnection *, xcb_generic_event_t *);
     void addPeekFunc(PeekFunc f);
 
-    // Peek at all queued events
-    qint32 generatePeekerId();
-    bool removePeekerId(qint32 peekerId);
-    enum PeekOption { PeekDefault = 0, PeekFromCachedIndex = 1 }; // see qx11info_x11.h
-    Q_DECLARE_FLAGS(PeekOptions, PeekOption)
-    typedef bool (*PeekerCallback)(xcb_generic_event_t *event, void *peekerData);
-    bool peekEventQueue(PeekerCallback peeker, void *peekerData = nullptr,
-                        PeekOptions option = PeekDefault, qint32 peekerId = -1);
-
     inline xcb_timestamp_t time() const { return m_time; }
     inline void setTime(xcb_timestamp_t t) { if (t > m_time) m_time = t; }
 
     inline xcb_timestamp_t netWmUserTime() const { return m_netWmUserTime; }
     inline void setNetWmUserTime(xcb_timestamp_t t) { if (t > m_netWmUserTime) m_netWmUserTime = t; }
 
-    bool hasXFixes() const { return has_xfixes; }
+    bool hasXFixes() const { return xfixes_first_event > 0; }
     bool hasXShape() const { return has_shape_extension; }
     bool hasXRandr() const { return has_randr_extension; }
     bool hasInputShape() const { return has_input_shape; }
     bool hasXKB() const { return has_xkb; }
-    bool hasXRender() const { return has_render_extension; }
-    bool hasXInput2() const { return m_xi2Enabled; }
 
+    bool supportsThreadedRendering() const { return m_reader->isRunning(); }
     bool threadedEventHandling() const { return m_reader->isRunning(); }
 
     xcb_timestamp_t getTimestamp();
-    xcb_window_t getSelectionOwner(xcb_atom_t atom) const;
-    xcb_window_t getQtSelectionOwner();
 
-    void setButtonState(Qt::MouseButton button, bool down) { m_buttonState.setFlag(button, down); }
-    Qt::MouseButtons buttonState() const { return m_buttonState; }
+    void setButton(Qt::MouseButton button, bool down) { if (down) m_buttons |= button; else m_buttons &= ~button; }
+    Qt::MouseButtons buttons() const { return m_buttons; }
     Qt::MouseButton translateMouseButton(xcb_button_t s);
 
     QXcbWindow *focusWindow() const { return m_focusWindow; }
-    void setFocusWindow(QWindow *);
-    QXcbWindow *mouseGrabber() const { return m_mouseGrabber; }
-    void setMouseGrabber(QXcbWindow *);
-    QXcbWindow *mousePressWindow() const { return m_mousePressWindow; }
-    void setMousePressWindow(QXcbWindow *);
+    void setFocusWindow(QXcbWindow *);
 
     QByteArray startupId() const { return m_startupId; }
     void setStartupId(const QByteArray &nextId) { m_startupId = nextId; }
@@ -461,35 +446,27 @@ public:
 
     QXcbNativeInterface *nativeInterface() const { return m_nativeInterface; }
 
-    QXcbSystemTrayTracker *systemTrayTracker() const;
-    static bool xEmbedSystemTrayAvailable();
-    static bool xEmbedSystemTrayVisualHasAlphaChannel();
+    QXcbSystemTrayTracker *systemTrayTracker();
 
-#if QT_CONFIG(xinput2)
-    void xi2SelectStateEvents();
-    void xi2SelectDeviceEvents(xcb_window_t window);
-    void xi2SelectDeviceEventsCompatibility(xcb_window_t window);
-    bool xi2SetMouseGrabEnabled(xcb_window_t w, bool grab);
-    bool xi2MouseEventsDisabled() const;
-    bool isAtLeastXI21() const { return m_xi2Enabled && m_xi2Minor >= 1; }
-    bool isAtLeastXI22() const { return m_xi2Enabled && m_xi2Minor >= 2; }
-    Qt::MouseButton xiToQtMouseButton(uint32_t b);
-#ifdef XCB_USE_XINPUT21
-    void xi2UpdateScrollingDevices();
+#ifdef XCB_USE_XINPUT2
+    void handleEnterEvent(const xcb_enter_notify_event_t *);
 #endif
+
 #ifdef XCB_USE_XINPUT22
-    bool startSystemResizeForTouchBegin(xcb_window_t window, const QPoint &point, Qt::Corner corner);
-    bool isTouchScreen(int id);
-#endif
+    bool xi2SetMouseGrabEnabled(xcb_window_t w, bool grab);
 #endif
+    Qt::MouseButton xiToQtMouseButton(uint32_t b);
+
     QXcbEventReader *eventReader() const { return m_reader; }
 
     bool canGrab() const { return m_canGrabServer; }
 
     QXcbGlIntegration *glIntegration() const { return m_glIntegration; }
 
+    bool xi2MouseEvents() const;
+
 protected:
-    bool event(QEvent *e) override;
+    bool event(QEvent *e) Q_DECL_OVERRIDE;
 
 public slots:
     void flush() { xcb_flush(m_connection); }
@@ -503,184 +480,147 @@ private:
     void initializeXFixes();
     void initializeXRender();
     void initializeXRandr();
-    void initializeXinerama();
     void initializeXShape();
     void initializeXKB();
     void handleClientMessageEvent(const xcb_client_message_event_t *event);
-    QXcbScreen* findScreenForCrtc(xcb_window_t rootWindow, xcb_randr_crtc_t crtc) const;
-    QXcbScreen* findScreenForOutput(xcb_window_t rootWindow, xcb_randr_output_t output) const;
-    QXcbVirtualDesktop* virtualDesktopForRootWindow(xcb_window_t rootWindow) const;
-    void updateScreens(const xcb_randr_notify_event_t *event);
+    QXcbScreen* createScreen(QXcbVirtualDesktop *virtualDesktop,
+                             xcb_randr_output_t outputId = XCB_NONE,
+                             xcb_randr_get_output_info_reply_t *output = 0);
+    QXcbScreen* findScreenForCrtc(xcb_window_t rootWindow, xcb_randr_crtc_t crtc);
+    QXcbScreen* findScreenForOutput(xcb_window_t rootWindow, xcb_randr_output_t output);
+    QXcbVirtualDesktop* virtualDesktopForRootWindow(xcb_window_t rootWindow);
     bool checkOutputIsPrimary(xcb_window_t rootWindow, xcb_randr_output_t output);
-    void updateScreen(QXcbScreen *screen, const xcb_randr_output_change_t &outputChange);
-    QXcbScreen *createScreen(QXcbVirtualDesktop *virtualDesktop,
-                             const xcb_randr_output_change_t &outputChange,
-                             xcb_randr_get_output_info_reply_t *outputInfo);
-    void destroyScreen(QXcbScreen *screen);
     void initializeScreens();
-    bool compressEvent(xcb_generic_event_t *event, int currentIndex, QXcbEventArray *eventqueue) const;
+    void updateScreens(const xcb_randr_notify_event_t *event);
 
-    bool m_xi2Enabled = false;
-#if QT_CONFIG(xinput2)
-    int m_xi2Minor = -1;
+    bool m_xi2Enabled;
+    int m_xi2Minor;
+#ifdef XCB_USE_XINPUT2
     void initializeXInput2();
-    void xi2SetupDevice(void *info, bool removeExisting = true);
+    void finalizeXInput2();
     void xi2SetupDevices();
-    struct TouchDeviceData {
-        QTouchDevice *qtTouchDevice = nullptr;
-        QHash<int, QWindowSystemInterface::TouchPoint> touchPoints;
-        QHash<int, QPointF> pointPressedPosition; // in screen coordinates where each point was pressed
-        struct ValuatorClassInfo {
-            double min = 0;
-            double max = 0;
-            int number = -1;
-            QXcbAtom::Atom label;
-        };
-        QVector<ValuatorClassInfo> valuatorInfo;
-
-        // Stuff that is relevant only for touchpads
-        QPointF firstPressedPosition;        // in screen coordinates where the first point was pressed
-        QPointF firstPressedNormalPosition;  // device coordinates (0 to 1, 0 to 1) where the first point was pressed
-        QSizeF size;                         // device size in mm
-        bool providesTouchOrientation = false;
-    };
-    TouchDeviceData *populateTouchDevices(void *info);
-    TouchDeviceData *touchDeviceForId(int id);
+    XInput2TouchDeviceData *touchDeviceForId(int id);
     void xi2HandleEvent(xcb_ge_event_t *event);
-    void xi2HandleHierarchyEvent(void *event);
+    void xi2HandleHierachyEvent(void *event);
     void xi2HandleDeviceChangedEvent(void *event);
     int m_xiOpCode, m_xiEventBase, m_xiErrorBase;
 #ifdef XCB_USE_XINPUT22
     void xi2ProcessTouch(void *xiDevEvent, QXcbWindow *platformWindow);
 #endif // XCB_USE_XINPUT22
-#if QT_CONFIG(tabletevent)
+#ifndef QT_NO_TABLETEVENT
     struct TabletData {
-        int deviceId = 0;
-        QTabletEvent::PointerType pointerType = QTabletEvent::UnknownPointer;
-        QTabletEvent::TabletDevice tool = QTabletEvent::Stylus;
-        Qt::MouseButtons buttons = 0;
-        qint64 serialId = 0;
-        bool inProximity = false;
+        TabletData() : deviceId(0), pointerType(QTabletEvent::UnknownPointer),
+            tool(QTabletEvent::Stylus), buttons(0), serialId(0), inProximity(false) { }
+        int deviceId;
+        QTabletEvent::PointerType pointerType;
+        QTabletEvent::TabletDevice tool;
+        Qt::MouseButtons buttons;
+        qint64 serialId;
+        bool inProximity;
         struct ValuatorClassInfo {
-            double minVal = 0;
-            double maxVal = 0;
-            double curVal = 0;
-            int number = -1;
+            ValuatorClassInfo() : minVal(0.), maxVal(0.), curVal(0.) { }
+            double minVal;
+            double maxVal;
+            double curVal;
+            int number;
         };
         QHash<int, ValuatorClassInfo> valuatorInfo;
     };
-    friend class QTypeInfo<TabletData>;
-    friend class QTypeInfo<TabletData::ValuatorClassInfo>;
-    bool xi2HandleTabletEvent(const void *event, TabletData *tabletData);
-    void xi2ReportTabletEvent(const void *event, TabletData *tabletData);
+    bool xi2HandleTabletEvent(void *event, TabletData *tabletData, QXcbWindowEventListener *eventListener);
+    void xi2ReportTabletEvent(TabletData &tabletData, void *event);
     QVector<TabletData> m_tabletData;
-    TabletData *tabletDataForDevice(int id);
-#endif // QT_CONFIG(tabletevent)
+#endif // !QT_NO_TABLETEVENT
     struct ScrollingDevice {
-        int deviceId = 0;
-        int verticalIndex = 0;
-        int horizontalIndex = 0;
-        double verticalIncrement = 0;
-        double horizontalIncrement = 0;
-        Qt::Orientations orientations = 0;
-        Qt::Orientations legacyOrientations = 0;
+        ScrollingDevice() : deviceId(0), verticalIndex(0), horizontalIndex(0), orientations(0), legacyOrientations(0) { }
+        int deviceId;
+        int verticalIndex, horizontalIndex;
+        double verticalIncrement, horizontalIncrement;
+        Qt::Orientations orientations;
+        Qt::Orientations legacyOrientations;
         QPointF lastScrollPosition;
     };
-    QHash<int, ScrollingDevice> m_scrollingDevices;
-#ifdef XCB_USE_XINPUT21
+    void updateScrollingDevice(ScrollingDevice& scrollingDevice, int num_classes, void *classes);
     void xi2HandleScrollEvent(void *event, ScrollingDevice &scrollingDevice);
-    void xi2UpdateScrollingDevice(ScrollingDevice &scrollingDevice);
-    ScrollingDevice *scrollingDeviceForId(int id);
+    QHash<int, ScrollingDevice> m_scrollingDevices;
+
+    static bool xi2GetValuatorValueIfSet(void *event, int valuatorNum, double *value);
+    static bool xi2PrepareXIGenericDeviceEvent(xcb_ge_event_t *event, int opCode);
 #endif
 
-    static bool xi2GetValuatorValueIfSet(const void *event, int valuatorNum, double *value);
-    static void xi2PrepareXIGenericDeviceEvent(xcb_ge_event_t *event);
-#endif
-
-    xcb_connection_t *m_connection = nullptr;
-    const xcb_setup_t *m_setup = nullptr;
-    const bool m_canGrabServer;
-    const xcb_visualid_t m_defaultVisualId;
+    xcb_connection_t *m_connection;
+    const xcb_setup_t *m_setup;
+    bool m_canGrabServer;
+    xcb_visualid_t m_defaultVisualId;
 
     QList<QXcbVirtualDesktop *> m_virtualDesktops;
     QList<QXcbScreen *> m_screens;
-    int m_primaryScreenNumber = 0;
+    int m_primaryScreenNumber;
 
     xcb_atom_t m_allAtoms[QXcbAtom::NAtoms];
 
-    xcb_timestamp_t m_time = XCB_CURRENT_TIME;
-    xcb_timestamp_t m_netWmUserTime = XCB_CURRENT_TIME;
+    xcb_timestamp_t m_time;
+    xcb_timestamp_t m_netWmUserTime;
 
     QByteArray m_displayName;
 
-    QXcbKeyboard *m_keyboard = nullptr;
+    QXcbKeyboard *m_keyboard;
 #ifndef QT_NO_CLIPBOARD
-    QXcbClipboard *m_clipboard = nullptr;
+    QXcbClipboard *m_clipboard;
 #endif
 #ifndef QT_NO_DRAGANDDROP
-    QXcbDrag *m_drag = nullptr;
+    QXcbDrag *m_drag;
 #endif
     QScopedPointer<QXcbWMSupport> m_wmSupport;
-    QXcbNativeInterface *m_nativeInterface = nullptr;
+    QXcbNativeInterface *m_nativeInterface;
 
-#if QT_CONFIG(xcb_xlib)
-    void *m_xlib_display = nullptr;
+#if defined(XCB_USE_XLIB)
+    void *m_xlib_display;
+#endif
+    QXcbEventReader *m_reader;
+#if defined(XCB_USE_XINPUT2)
+    QHash<int, QWindowSystemInterface::TouchPoint> m_touchPoints;
+    QHash<int, XInput2TouchDeviceData*> m_touchDevices;
+#endif
+#ifdef Q_XCB_DEBUG
+    struct CallInfo {
+        int sequence;
+        QByteArray file;
+        int line;
+    };
+    QVector<CallInfo> m_callLog;
+    QMutex m_callLogMutex;
+    void log(const char *file, int line, int sequence);
+    template <typename cookie_t>
+    friend cookie_t q_xcb_call_template(const cookie_t &cookie, QXcbConnection *connection, const char *file, int line);
 #endif
-    QXcbEventReader *m_reader = nullptr;
 
-#if QT_CONFIG(xinput2)
-    QHash<int, TouchDeviceData> m_touchDevices;
-#ifdef XCB_USE_XINPUT22
-    struct StartSystemResizeInfo {
-        xcb_window_t window = XCB_NONE;
-        uint16_t deviceid;
-        uint32_t pointid;
-        Qt::Corner corner;
-    } m_startSystemResizeInfo;
-#endif
-#endif
     WindowMapper m_mapper;
 
     QVector<PeekFunc> m_peekFuncs;
 
-    uint32_t xfixes_first_event = 0;
-    uint32_t xrandr_first_event = 0;
-    uint32_t xkb_first_event = 0;
+    uint32_t xfixes_first_event;
+    uint32_t xrandr_first_event;
+    uint32_t xkb_first_event;
 
-    bool has_xfixes = false;
-    bool has_xinerama_extension = false;
-    bool has_shape_extension = false;
-    bool has_randr_extension = false;
+    bool has_shape_extension;
+    bool has_randr_extension;
     bool has_input_shape;
-    bool has_xkb = false;
-    bool has_render_extension = false;
+    bool has_xkb;
 
-    Qt::MouseButtons m_buttonState = 0;
+    Qt::MouseButtons m_buttons;
 
-    QXcbWindow *m_focusWindow = nullptr;
-    QXcbWindow *m_mouseGrabber = nullptr;
-    QXcbWindow *m_mousePressWindow = nullptr;
+    QXcbWindow *m_focusWindow;
 
-    xcb_window_t m_clientLeader = 0;
     QByteArray m_startupId;
-    QXcbSystemTrayTracker *m_systemTrayTracker = nullptr;
-    QXcbGlIntegration *m_glIntegration = nullptr;
-    bool m_xiGrab = false;
+    QXcbSystemTrayTracker *m_systemTrayTracker;
+    QXcbGlIntegration *m_glIntegration;
+    bool m_xiGrab;
 
-    xcb_window_t m_qtSelectionOwner = 0;
-
-    bool m_mainEventLoopFlushedQueue = false;
-    qint32 m_peekerIdSource = 0;
-    bool m_peekerIndexCacheDirty = false;
-    QHash<qint32, qint32> m_peekerToCachedIndex;
     friend class QXcbEventReader;
 };
-#if QT_CONFIG(xinput2)
-#if QT_CONFIG(tabletevent)
-Q_DECLARE_TYPEINFO(QXcbConnection::TabletData::ValuatorClassInfo, Q_PRIMITIVE_TYPE);
-Q_DECLARE_TYPEINFO(QXcbConnection::TabletData, Q_MOVABLE_TYPE);
-#endif
-#endif
+
+#define DISPLAY_FROM_XCB(object) ((Display *)(object->connection()->xlib_display()))
+#define CREATE_VISUALINFO_FROM_DEFAULT_VISUALID(object) ((XVisualInfo *)(object->connection()->createVisualInfoForDefaultVisualId()))
 
 template<typename T>
 xcb_generic_event_t *QXcbConnection::checkEvent(T &checker)
@@ -709,32 +649,21 @@ private:
     QXcbConnection *m_connection;
 };
 
-#define Q_XCB_REPLY_CONNECTION_ARG(connection, ...) connection
-
-#define Q_XCB_REPLY(call, ...) \
-    std::unique_ptr<call##_reply_t, decltype(std::free) *>( \
-        call##_reply(Q_XCB_REPLY_CONNECTION_ARG(__VA_ARGS__), call(__VA_ARGS__), nullptr), \
-        std::free \
-    )
-
-#define Q_XCB_REPLY_UNCHECKED(call, ...) \
-    std::unique_ptr<call##_reply_t, decltype(std::free) *>( \
-        call##_reply(Q_XCB_REPLY_CONNECTION_ARG(__VA_ARGS__), call##_unchecked(__VA_ARGS__), nullptr), \
-        std::free \
-    )
-
-template <typename T>
-union q_padded_xcb_event {
-  T event;
-  char padding[32];
-};
-
-// The xcb_send_event() requires all events to have 32 bytes. It calls memcpy() on the
-// passed in event. If the passed in event is less than 32 bytes, memcpy() reaches into
-// unrelated memory.
-#define Q_DECLARE_XCB_EVENT(event_var, event_type) \
-    q_padded_xcb_event<event_type> store = {}; \
-    auto &event_var = store.event;
+#ifdef Q_XCB_DEBUG
+template <typename cookie_t>
+cookie_t q_xcb_call_template(const cookie_t &cookie, QXcbConnection *connection, const char *file, int line)
+{
+    connection->log(file, line, cookie.sequence);
+    return cookie;
+}
+#define Q_XCB_CALL(x) q_xcb_call_template(x, connection(), __FILE__, __LINE__)
+#define Q_XCB_CALL2(x, connection) q_xcb_call_template(x, connection, __FILE__, __LINE__)
+#define Q_XCB_NOOP(c) q_xcb_call_template(xcb_no_operation(c->xcb_connection()), c, __FILE__, __LINE__);
+#else
+#define Q_XCB_CALL(x) x
+#define Q_XCB_CALL2(x, connection) x
+#define Q_XCB_NOOP(c) (void)c;
+#endif
 
 QT_END_NAMESPACE
 
